#!/bin/csh -f
#
#   Control script to monitor Frealign jobs
#
set working_directory	= `pwd`
set SCRATCH		= `grep scratch_dir mparameters | awk '{print $2}'`
if ( $status || $SCRATCH == "" ) then
  set SCRATCH		= ${working_directory}/scratch
endif
if ( ! -d $SCRATCH ) then
  mkdir $SCRATCH
endif

set bin_dir		= `grep frealign_bin_dir $SCRATCH/mparameters_run | awk '{print $2}'`
if ( $status || $bin_dir == "" ) then
  set bin_dir		= `which frealign_v9.exe`
  set bin_dir		= ${bin_dir:h}
endif
set cluster_type	= `grep cluster_type $SCRATCH/mparameters_run | awk '{print $2}'`
set cpid		= ${1}
set restart		= `grep restart_after_crash $SCRATCH/mparameters_run | awk '{print $2}'`

if (`echo ${cluster_type} | tr '[a-z]' '[A-Z]'` == "SGE") then
  qstat >& /dev/null
  if ($status) then
    echo "No Sun Grid Engine available. Switching to non-SGE mode." >> frealign.log
    set cluster_type = "none"
  endif
endif

wait_for_pid:

sleep 10

monitorloop:

if ( ! -e $SCRATCH/pid.log ) goto wait_for_pid

grep term $SCRATCH/pid.log >& /dev/null
if ( ! $status ) then
  echo "Normal termination of frealign run "`date` >> frealign.log
  set restart = "F"
  goto terminate
endif

grep kill $SCRATCH/pid.log >& /dev/null
if ( ! $status ) then
  echo "Killing frealign run..." >> frealign.log
  set restart = "F"
  goto terminate
endif

ps ${cpid} >& /dev/null
if ( $status ) then
  sleep 5
  grep term $SCRATCH/pid.log >& /dev/null
  if ( $status ) then
    grep "monitor reconstruct" $SCRATCH/pid.log >> /dev/null
    if ( $status ) then
      echo "Frealign run script crashed "`date` >> frealign.log
      echo "Terminating..." >> frealign.log
    else
      echo "Normal termination of frealign run "`date` >> frealign.log
      set restart = "F"
    endif
    goto terminate
  endif
endif

foreach pid (`grep -v monitor $SCRATCH/pid.log | awk '{print $1}'`)

  if (`echo ${cluster_type} | tr '[a-z]' '[A-Z]'` == "SGE") then
    qstat -j $pid >& /dev/null
    set err = $status
  else
    ps $pid >& /dev/null
    set err = $status
  endif
  if ($err) then
    grep term $SCRATCH/pid.log >& /dev/null
    if ( ! $status ) then
      echo "Normal termination of frealign run "`date` >> frealign.log
      set restart = "F"
      goto terminate
    endif
    sleep 5
    grep $pid $SCRATCH/pid.log >& /dev/null
    if ( ! $status ) then
      set logfile = `grep $pid $SCRATCH/pid.log | awk '{print $2}'`
      grep "Normal termination" $SCRATCH/${logfile} >& /dev/null
      if ($status) then
        echo "Job $pid crashed." >> frealign.log
        sleep 5
        echo "Logfile "$SCRATCH/${logfile} >> frealign.log
        echo "Final lines:" >> frealign.log
        echo >> frealign.log
        tail $SCRATCH/${logfile} >> frealign.log
        echo "Terminating..." >> frealign.log
        goto terminate
      endif
    endif
  endif

end

sleep 2

goto monitorloop

terminate:

if (`echo ${cluster_type} | tr '[a-z]' '[A-Z]'` == "SGE") then
  qdel `grep -v monitor $SCRATCH/pid.log | awk '{print $1}'` >& /dev/null
  kill `pstree -p ${cpid} | sed 's/(/\n(/g' | grep '(' | sed 's/(\(.*\)).*/\1/' | tr "\n" " "`
else
  kill `pstree -p ${cpid} | sed 's/(/\n(/g' | grep '(' | sed 's/(\(.*\)).*/\1/' | tr "\n" " "`
  kill -9 `grep -v monitor $SCRATCH/pid.log | awk '{print $1}'` >& /dev/null
endif

sleep 10
if ( $restart == "T" ) then
  grep "monitor refine" $SCRATCH/pid.log >> /dev/null
  if ( ! $status ) then
    echo "Restarting refinement..." >> frealign.log
    mv frealign.log frealign_crash.log
    ${bin_dir}/frealign_run_refine restart
  end
  grep "monitor search" $SCRATCH/pid.log >> /dev/null
  if ( ! $status ) then
    echo "Restarting search..." >> frealign.log
    mv frealign.log frealign_crash.log
    ${bin_dir}/frealign_run_search restart
  end
endif
