#!/bin/csh -f
#
#   Control script to submit multiple jobs on a cluster using
#   the Sun Grid Engine, or on a multi-core machine.
#
set working_directory	= `pwd`
set SCRATCH		= `grep scratch_dir mparameters | awk '{print $2}'`
if ( $status || $SCRATCH == "" ) then
  set SCRATCH		= ${working_directory}/scratch
endif
if ( ! -d $SCRATCH ) then
  mkdir $SCRATCH
endif

cp mparameters $SCRATCH/mparameters_run

set first		= ${1}
set last		= ${2}
set start		= ${3}
set data_input		= `grep data_input $SCRATCH/mparameters_run | awk '{print $2}'`
set nclass		= `grep nclasses $SCRATCH/mparameters_run | awk '{print $2}'`
set bin_dir		= `grep frealign_bin_dir $SCRATCH/mparameters_run | awk '{print $2}'`
if ( $status || $bin_dir == "" ) then
  set bin_dir		= `which frealign_v9.exe`
  set bin_dir		= ${bin_dir:h}
endif
set cluster_type	= `grep cluster_type $SCRATCH/mparameters_run | awk '{print $2}'`
set night_queue		= `grep night_queue $SCRATCH/mparameters_run | awk '{print $2}'`
set stn			= `grep qsub_string_rec mparameters | awk -F\" '{print $2}'`
set no_delete           = `grep delete_scratch $SCRATCH/mparameters_run | awk '{print $2}'`
if ( $no_delete == "F" ) then
  set no_delete = 1
else
  set no_delete = 0
endif

set raw_images		= `grep raw_images_low $SCRATCH/mparameters_run | awk '{print $2}'`
if ( $raw_images == "" ) set raw_images = `grep raw_images $SCRATCH/mparameters_run | awk '{print $2}'`
set raw_images = `echo ${raw_images:r}`
set extension = `ls $raw_images.* | head -1`
if ( $extension == "" ) then
  set raw_images = ${working_directory}/${raw_images}
  set extension = `ls $raw_images.* | head -1`
endif
set extension = `echo ${extension:e}`

set nx = `${bin_dir}/fheader.exe ${raw_images}.${extension} | grep --binary-files=text NX | awk '{print $4}'`
set mem_small = `echo $nx | awk '{print int(10 * $1^3 * 4 * 3 /1024^3 + 1)/10}'`  
set mem_small = `echo $mem_small | awk '{if ($1 < 1) {print 1} else {print $1} }'`

if (`echo ${cluster_type} | tr '[a-z]' '[A-Z]'` == "LSF") then
 set mem_small = `echo $mem_small | awk '{print int($1 * 1024)}'`
endif

if (`echo ${cluster_type} | tr '[a-z]' '[A-Z]'` == "SGE") then
  qstat >& /dev/null
  if ($status) then
#    echo "No Sun Grid Engine available. Switching to non-SGE mode." >> frealign.log
    set cluster_type = "none"
  endif
endif

mainloop:

cp mparameters $SCRATCH/mparameters_run
set nproc		= `grep nprocessor_rec $SCRATCH/mparameters_run | awk '{print $2}'`
set sym			= `grep Symmetry $SCRATCH/mparameters_run | awk '{print $2}'`

@ incr = $last - $first + 1
set incr = `expr ${incr} \* ${nclass} / ${nproc}`
@ incr++

reconstruct:
# echo "Calculating 3D structure...." >> frealign.log

set h = `date +%k`
set nq = ""
if ($h > 18) set nq = "-l night=true"
if ($h < 6) set nq = "-l night=true"
if ($night_queue != "T" ) set nq = ""

\rm $SCRATCH/pid_temp.log >& /dev/null
set npart = 0
set firstn = $first
@ lastn = $first + $incr - 1
if ( $lastn >= $last ) set lastn = $last

while ( $lastn <= $last )
  echo "Cycle "$start": reconstructing particles $firstn to $lastn on "`date` >> frealign.log
  @ npart++

  set nc = 1
  while ( $nc <= $nclass )
    \rm $SCRATCH/${data_input}_mult_reconstruct_r${nc}_n${firstn}.log >& /dev/null
    \rm $SCRATCH/${data_input}_${start}_r${nc}.shft_* >& /dev/null
    \rm $SCRATCH/${data_input}_mult_refine_n_r${nc}.log_* >& /dev/null
    if (`echo ${cluster_type} | tr '[a-z]' '[A-Z]'` == "SGE") then
      if ( $sym == "H" || $sym == "HP" ) then
        set pid = `qsub $stn -l mem_free=${mem_small}G $nq -cwd -r y -V -terse -e $SCRATCH/stderr -o $SCRATCH/stderr -N r${nc}_${firstn} ${bin_dir}/mult_hreconstruct_n.com $firstn $lastn $start $nc`
      else
        set pid = `qsub $stn -l mem_free=${mem_small}G $nq -cwd -r y -V -terse -e $SCRATCH/stderr -o $SCRATCH/stderr -N r${nc}_${firstn} ${bin_dir}/mult_reconstruct_n.com $firstn $lastn $start $nc`
      endif
    else if (`echo ${cluster_type} | tr '[a-z]' '[A-Z]'` == "LSF") then 
      set pid = cycle_${firstn}_${start}_${nc}
      if ( $sym == "H" || $sym == "HP" ) then
        bsub -J $pid -W 12:00 -q short -R "select[model!=Opteron2216]" -R "rusage[mem=${mem_small}]" -cwd  "$working_directory" -r -e $SCRATCH/stderr -o $SCRATCH/stderr ${bin_dir}/mult_hreconstruct_n.com $firstn $lastn $start $nc
      else
        bsub -J $pid -W 12:00 -q short -R "select[model!=Opteron2216]" -R "rusage[mem=${mem_small}]" -cwd  "$working_directory" -r -e $SCRATCH/stderr -o $SCRATCH/stderr ${bin_dir}/mult_reconstruct_n.com $firstn $lastn $start $nc
      endif
    else
      if ( $sym == "H" || $sym == "HP" ) then
        ${bin_dir}/mult_hreconstruct_n.com $firstn $lastn $start $nc &
      else
        ${bin_dir}/mult_reconstruct_n.com $firstn $lastn $start $nc &
      endif
      set pid = $!
    endif
    echo $pid ${data_input}_mult_reconstruct_r${nc}_n${firstn}.log >> $SCRATCH/pid_temp.log
    @ nc++
  end

#  sleep 2

  if ( $lastn == $last ) then
    goto submission_done
  endif
  @ firstn = $firstn + $incr
  @ lastn = $lastn + $incr
  if ( $firstn >= $last ) set firstn = $last
  if ( $lastn >= $last ) set lastn = $last
end

submission_done:

sleep 10

cat $SCRATCH/pid_temp.log >> $SCRATCH/pid.log

set nc = 1
while ( $nc <= $nclass )

  set firstn = $first
  @ lastn = $first + $incr - 1
  if ( $lastn >= $last ) set lastn = $last

checkdone_rec:

  sleep 5
  while ( $firstn <= $last )

    grep --binary-files=text finished $SCRATCH/${data_input}_mult_reconstruct_r${nc}_n${firstn}.log >& /dev/null

    if ($status) goto checkdone_rec

    echo "Cycle "$start": reconstruction for particles $firstn to $lastn, ref $nc, finished "`date` >> frealign.log
    set pid = `grep "${data_input}_mult_reconstruct_r${nc}_n${firstn}.log" $SCRATCH/pid.log | awk '{print $1}'`
    sed -i "/$pid/d" $SCRATCH/pid.log

    @ firstn = $firstn + $incr
    @ lastn = $lastn + $incr
    if ( $lastn >= $last ) set lastn = $last
  end

  @ nc++

end

set h = `date +%k`
set nq = ""
if ($h > 18) set nq = "-l night=true"
if ($h < 6) set nq = "-l night=true"
if ($night_queue != "T" ) set nq = ""

\rm $SCRATCH/pid_temp.log >& /dev/null
set nc = 1
while ( $nc <= $nclass )

  echo "Cycle "$start": merging 3D dump files for ref $nc "`date` >> frealign.log
  echo $npart > $SCRATCH/merge_3d_r${nc}.in
  set firstn = $first
  @ lastn = $first + $incr - 1
  while ( $firstn <= $last )

    echo $SCRATCH/${data_input}_${start}_r${nc}_n${firstn}.${extension} >> $SCRATCH/merge_3d_r${nc}.in

    @ firstn = $firstn + $incr
    @ lastn = $lastn + $incr
    if ( $lastn >= $last ) set lastn = $last

  end

  echo $SCRATCH/${data_input}_${start}_r${nc}.res >> $SCRATCH/merge_3d_r${nc}.in
  echo $SCRATCH/${data_input}_${start}_r${nc}.${extension} >> $SCRATCH/merge_3d_r${nc}.in
  echo $SCRATCH/${data_input}_${start}_r${nc}_weights >> $SCRATCH/merge_3d_r${nc}.in
  echo $SCRATCH/${data_input}_${start}_r${nc}_map1.${extension} >> $SCRATCH/merge_3d_r${nc}.in
  echo $SCRATCH/${data_input}_${start}_r${nc}_map2.${extension} >> $SCRATCH/merge_3d_r${nc}.in
  echo $SCRATCH/${data_input}_${start}_r${nc}_phasediffs >> $SCRATCH/merge_3d_r${nc}.in
  echo $SCRATCH/${data_input}_${start}_r${nc}_pointspread >> $SCRATCH/merge_3d_r${nc}.in

  cp $SCRATCH/${data_input}_${start}_r${nc}_n${first}.res $SCRATCH/${data_input}_${start}_r${nc}.res

  if (`echo ${cluster_type} | tr '[a-z]' '[A-Z]'` == "SGE") then
    set pid = `qsub $stn -l mem_free=${mem_small}G $nq -cwd -r y -V -terse -e $SCRATCH/stderr -o $SCRATCH/stderr -N m${nc} ${bin_dir}/merge_3d.com ${nc}`
  else if (`echo ${cluster_type} | tr '[a-z]' '[A-Z]'` == "LSF") then 
    set pid = cycle_${start}_${nc}
    bsub -J $pid -W 12:00 -q short -R "select[model!=Opteron2216]" -R "rusage[mem=${mem_small}]" -cwd  "$working_directory" -r -e $SCRATCH/stderr -o $SCRATCH/stderr ${bin_dir}/merge_3d.com ${nc}
  else
    ${bin_dir}/merge_3d.com ${nc} &
    set pid = $!
  endif

  echo $pid merge_3d_r${nc}.log >> $SCRATCH/pid_temp.log
  @ nc++

end
  
sleep 10

cat $SCRATCH/pid_temp.log >> $SCRATCH/pid.log

set nc = 1
while ( $nc <= $nclass )
  
checkdonem:

  sleep 2

  grep --binary-files=text "merge_3d.com finished" $SCRATCH/merge_3d_r${nc}.log >& /dev/null
  if ($status) goto checkdonem

  ls $SCRATCH/${data_input}_${start}_r${nc}.${extension} >& /dev/null
  if ($status) goto checkdonem

  ls $SCRATCH/${data_input}_${start}_r${nc}.res >& /dev/null
  if ($status) goto checkdonem

  set pid = `grep "merge_3d_r${nc}.log" $SCRATCH/pid.log | awk '{print $1}'`
  sed -i "/$pid/d" $SCRATCH/pid.log

  sleep 1

  echo "Cycle "$start": merging for ref $nc finished "`date` >> frealign.log
  cat $SCRATCH/${data_input}_${start}_r${nc}.res >> ${working_directory}/${data_input}_${start}_r${nc}.par
  mv $SCRATCH/${data_input}_${start}_r${nc}.${extension} ${working_directory}/${data_input}_${start}_r${nc}.${extension}
  if ( ! $no_delete ) then
    \rm $SCRATCH/${data_input}_${start}_r${nc}.res
    \rm $SCRATCH/${data_input}_${start}_r${nc}.par
  endif

  @ nc++

end

# Clean up

@ prev = $start - 1

if ( ! $no_delete ) then
  set nc = 1
  while ( $nc <= $nclass )

    set firstn = $first
    @ lastn = $first + $incr - 1
    while ( $firstn <= $last )

      \rm $SCRATCH/${data_input}_${start}_r${nc}_n${firstn}.${extension}
      \rm $SCRATCH/${data_input}_${start}_r${nc}_n${firstn}.res

      @ firstn = $firstn + $incr
      @ lastn = $lastn + $incr
      if ( $lastn >= $last ) set lastn = $last

    end

    \rm $SCRATCH/${data_input}_${prev}_r${nc}_map1.${extension} >& /dev/null
    \rm $SCRATCH/${data_input}_${prev}_r${nc}_map2.${extension} >& /dev/null
    \rm $SCRATCH/${data_input}_${prev}_r${nc}_weights >& /dev/null
    @ nc++

  end
endif

echo "Cycle "$start": reconstruction done." >> frealign.log

date
