#!/bin/csh -f
#
#   Control script to submit multiple jobs on a cluster using
#   the Sun Grid Engine, or on a multi-core machine.
#
set working_directory	= `pwd`
set SCRATCH		= `grep scratch_dir mparameters | awk '{print $2}'`
if ( $status || $SCRATCH == "" ) then
  set SCRATCH		= ${working_directory}/scratch
endif
if ( ! -d $SCRATCH ) then
  mkdir $SCRATCH
endif

cp mparameters $SCRATCH/mparameters_run

set start		= `grep start_process $SCRATCH/mparameters_run | awk '{print $2}'`
set end			= `grep end_process $SCRATCH/mparameters_run | awk '{print $2}'`
set first		= `grep first_particle $SCRATCH/mparameters_run | awk '{print $2}'`
set last		= `grep last_particle $SCRATCH/mparameters_run | awk '{print $2}'`
set data_input		= `grep data_input $SCRATCH/mparameters_run | awk '{print $2}'`
set bin_dir		= `grep frealign_bin_dir $SCRATCH/mparameters_run | awk '{print $2}'`
if ( $status || $bin_dir == "" ) then
  set bin_dir		= `which frealign_v9.exe`
  set bin_dir		= ${bin_dir:h}
endif
set cluster_type	= `grep cluster_type $SCRATCH/mparameters_run | awk '{print $2}'`
set night_queue		= `grep night_queue $SCRATCH/mparameters_run | awk '{print $2}'`
set reschedule		= `grep reschedule_if_qw $SCRATCH/mparameters_run | awk '{print $2}'`
set stn			= `grep qsub_string_ref mparameters | awk -F\" '{print $2}'`
set no_delete		= `grep delete_scratch $SCRATCH/mparameters_run | awk '{print $2}'`
if ( $no_delete == "F" ) then
  set no_delete = 1
else
  set no_delete = 0
endif

if (`echo ${cluster_type} | tr '[a-z]' '[A-Z]'` == "SGE") then
  qstat >& /dev/null
  if ($status) then
#    echo "No Sun Grid Engine available. Switching to non-SGE mode." >> frealign.log
    set cluster_type = "none"
  endif
endif

set raw_images		= `grep raw_images_high $SCRATCH/mparameters_run | awk '{print $2}'`
if ( $raw_images == "" ) set raw_images = `grep raw_images $SCRATCH/mparameters_run | awk '{print $2}'`
set raw_images = `echo ${raw_images:r}`
set extension = `ls $raw_images.* | head -1`
if ( $extension == "" ) then
  set raw_images = ${working_directory}/${raw_images}
  set extension = `ls $raw_images.* | head -1`
endif
set extension = `echo ${extension:e}`
set nx = `${bin_dir}/fheader.exe ${raw_images}.${extension} | grep --binary-files=text NX | awk '{print $4}'`
if ( $nx > 300 ) then
  set mem_big = `echo $nx | awk '{print int(10 * $1^3 * 4 * 3 /1024^3 + 1)/10}'`
else
  set mem_big = `echo $nx | awk '{print int(10 * $1^3 * 4 * 66 /1024^3 + 1)/10}'`
endif
set mem_big = `echo $mem_big | awk '{if ($1 < 1) {print 1} else {print $1} }'`
set mem_small = `echo $nx | awk '{print int(10 * $1^3 * 4 * 3 /1024^3 + 1)/10}'`
set mem_small = `echo $mem_small | awk '{if ($1 < 1) {print 1} else {print $1} }'`

if (`echo ${cluster_type} | tr '[a-z]' '[A-Z]'` == "LSF") then
 set mem_big = `echo $mem_big | awk '{print int($1 * 1024)}'`
 set mem_small = `echo $mem_small | awk '{print int($1 * 1024)}'`
endif

if ( $first == "" || $last == "" ) then
  set first = 1
  set last = `${bin_dir}/fheader.exe ${raw_images}.${extension} | grep --binary-files=text NX | awk '{print $6}'`
endif

echo "monitor refine" > $SCRATCH/pid.log

# Check what type of parameter file is available
set pix                 = `grep pix_size $SCRATCH/mparameters_run | awk '{print $2}'`
set dstep               = `grep dstep $SCRATCH/mparameters_run | awk '{print $2}'`
set sym                 = `grep Symmetry $SCRATCH/mparameters_run | awk '{print $2}'`
@ prev = $start - 1
set nc = `grep -v C ${working_directory}/${data_input}_${prev}_r1.par | head -1 | wc | awk '{print $2}'`
if ( $nc == 4 ) then
  echo "Found startup parameter file with film#, defocus 1, defocus 2, astig. angle." >> frealign.log
  set mag = `echo $dstep $pix | awk '{print int(10000 * $1 / $2)}'`
  echo "Setting magnification to "${mag}"." >> frealign.log
  mv ${data_input}_${prev}_r1.par ${data_input}_${prev}.par
  echo "C Startup parameters" > ${data_input}_${prev}_r1.par
  echo "C           PSI   THETA     PHI       SHX       SHY     MAG  FILM      DF1      DF2  ANGAST     OCC     -LogP      SIGMA   SCORE  CHANGE" >> ${data_input}_${prev}_r1.par
  if ( $sym == "H" || $sym == "HP" ) then
    grep -v C ${data_input}_${prev}.par | awk '{printf "%7d%8.2f%8.2f%8.2f%10.2f%10.2f%8d%6d%9.1f%9.1f%8.2f%7.2f%11d%11.4f%8.2f%8.2f\n",NR,0.0,90.0,0.0,0.0,0.0,'${mag}',$1,$2,$3,$4,100.0,0,0.5,100.0,0.0 }' >> ${data_input}_${prev}_r1.par
  else
    grep -v C ${data_input}_${prev}.par | awk '{printf "%7d%8.2f%8.2f%8.2f%10.2f%10.2f%8d%6d%9.1f%9.1f%8.2f%7.2f%11d%11.4f%8.2f%8.2f\n",NR,0.0,0.0,0.0,0.0,0.0,'${mag}',$1,$2,$3,$4,100.0,0,0.5,100.0,0.0 }' >> ${data_input}_${prev}_r1.par
  endif
endif

set nc = `grep -v C ${working_directory}/${data_input}_${prev}_r1.par | head -1 | wc | awk '{print $2}'`
if ( $nc < 12 ) then
  echo "ERROR: Something wrong with parameter file." >> frealign.log
  echo "If input file contained film#, defocus 1, defocus 2, astig. angle," >> frealign.log
  echo "please make sure that numbers are separated by spaces, not commas." >> frealign.log
  echo "monitor term" >> $SCRATCH/pid.log
  exit
endif

mainloop:

set nclass		= `grep nclasses $SCRATCH/mparameters_run | awk '{print $2}'`
@ prev = $start - 1

# Split into N classes if desired
set nc = `ls ${data_input}_${prev}_r?.par ${data_input}_${prev}_r??.par ${data_input}_${prev}_r???.par | wc -l`
if ($nc != $nclass) then
  ${bin_dir}/split_classes.com
endif

set nc = `ls ${data_input}_${prev}_r?.par ${data_input}_${prev}_r??.par ${data_input}_${prev}_r???.par | wc -l`
set i = `ls ${data_input}_${prev}_r?.${extension} ${data_input}_${prev}_r??.${extension} ${data_input}_${prev}_r???.${extension} | wc -l`
if ($nc != $i) then
  ${bin_dir}/mult_reconstruct.com $first $last $prev
endif

cp mparameters $SCRATCH/mparameters_run
set nproc		= `grep nprocessor_ref $SCRATCH/mparameters_run | awk '{print $2}'`
set itmax		= `grep ITMAX $SCRATCH/mparameters_run | awk '{print $2}'`
set refineshiftinc	= `grep refineshiftinc $SCRATCH/mparameters_run | awk '{print $2}'`
set refineangleinc	= `grep refineangleinc $SCRATCH/mparameters_run | awk '{print $2}'`
set mode		= `grep MODE $SCRATCH/mparameters_run | awk '{print $2}'`
set sym			= `grep Symmetry $SCRATCH/mparameters_run | awk '{print $2}'`

set shiftiter		= `expr ${start} % ${refineshiftinc}`
set angleiter		= `expr ${start} % ${refineangleinc}`

if ( $nclass < 2 ) then
  set shiftiter = 0
  set angleiter = 0
endif

@ incr = $last - $first + 1
set incr = `expr ${incr} \* ${nclass} / ${nproc}`
@ incr++

#if ( $mode != 1 ) then
#  if (${shiftiter} == 0) then  
#    if (${angleiter} == 0) then
#      set incr = `expr ${incr} / ${itmax} \* 3`
#    endif
#  endif
#endif

# queue submission loop

set h = `date +%k`
set nq = ""
if ($h > 18) set nq = "-l night=true"
if ($h < 6) set nq = "-l night=true"
if ($night_queue != "T" ) set nq = ""

\rm $SCRATCH/pid_temp.log >& /dev/null
subloop:
set firstn = $first
@ lastn = $first + $incr - 1
if ( $lastn >= $last ) set lastn = $last

while ( $lastn <= $last )
#  echo "Cycle "$start": refining particles" $firstn to $lastn on `date` >> frealign.log

  set nc = 1
  while ( $nc <= $nclass )

    set fst = $firstn
    if ( -e $SCRATCH/${data_input}_${start}_r${nc}.par_${firstn}_${lastn} ) then
      if ( $firstn == $first && $nc == 1 ) echo "Recovering from crashed run..." >> frealign.log
loop_recovery:
      set fst = `grep -v C $SCRATCH/${data_input}_${start}_r${nc}.par_${firstn}_${lastn} | tail -1 | awk '{print $1}'`
      if ( $fst != "" ) then
        @ fst++
        if ( -e $SCRATCH/${data_input}_${start}_r${nc}.par_${fst}_${lastn} ) then
          grep -v C $SCRATCH/${data_input}_${start}_r${nc}.par_${fst}_${lastn} >> $SCRATCH/${data_input}_${start}_r${nc}.par_${firstn}_${lastn}
          \rm $SCRATCH/${data_input}_${start}_r${nc}.par_${fst}_${lastn}
          goto loop_recovery
        endif
      else
        set fst = $firstn
        \rm $SCRATCH/${data_input}_${start}_r${nc}.par_${firstn}_${lastn}
      endif
    endif

      if ( $lastn >= $fst ) then
      echo "Cycle "$start": refining particles" $fst to $lastn", class "${nc}" "`date` >> frealign.log

    if (${shiftiter} == 0) then
      if (${angleiter} == 0) then
        if (`echo ${cluster_type} | tr '[a-z]' '[A-Z]'` == "SGE") then
          if ( $sym == "H" || $sym == "HP" ) then
            set pid = `qsub $stn -l mem_free=${mem_big}G $nq -cwd -r y -V -terse -e $SCRATCH/stderr -o $SCRATCH/stderr -N n${nc}_${fst} ${bin_dir}/mult_hrefine_n.com $fst $lastn $start $nc $mode 1 1 1 1 1`
          else
            set pid = `qsub $stn -l mem_free=${mem_big}G $nq -cwd -r y -V -terse -e $SCRATCH/stderr -o $SCRATCH/stderr -N n${nc}_${fst} ${bin_dir}/mult_refine_n.com $fst $lastn $start $nc $mode 1 1 1 1 1`
          endif
        else if (`echo ${cluster_type} | tr '[a-z]' '[A-Z]'` == "LSF") then 
          set pid = cycle_${firstn}_${start}_${nc}
          if ( $sym == "H" || $sym == "HP" ) then
            bsub -J $pid -W 12:00 -q short -R "select[model!=Opteron2216]" -R "rusage[mem=${mem_big}]" -cwd  "$working_directory" -r -e $SCRATCH/stderr -o $SCRATCH/stderr ${bin_dir}/mult_hrefine_n.com $firstn $lastn $start $nc $mode 1 1 1 1 1
          else
            bsub -J $pid -W 12:00 -q short -R "select[model!=Opteron2216]" -R "rusage[mem=${mem_big}]" -cwd  "$working_directory" -r -e $SCRATCH/stderr -o $SCRATCH/stderr ${bin_dir}/mult_refine_n.com $firstn $lastn $start $nc $mode 1 1 1 1 1
          endif
        else
          if ( $sym == "H" || $sym == "HP" ) then
            ${bin_dir}/mult_hrefine_n.com $fst $lastn $start $nc $mode 1 1 1 1 1 &
          else
            ${bin_dir}/mult_refine_n.com $fst $lastn $start $nc $mode 1 1 1 1 1 &
          endif
          set pid = $!
        endif
      else if (${angleiter} != 0) then
        if (`echo ${cluster_type} | tr '[a-z]' '[A-Z]'` == "SGE") then
          if ( $sym == "H" || $sym == "HP" ) then
            set pid = `qsub $stn -l mem_free=${mem_big}G $nq -cwd -r y -V -terse -e $SCRATCH/stderr -o $SCRATCH/stderr -N n${nc}_${fst} ${bin_dir}/mult_hrefine_n.com $fst $lastn $start $nc 1 0 0 0 1 1`
          else
            set pid = `qsub $stn -l mem_free=${mem_big}G $nq -cwd -r y -V -terse -e $SCRATCH/stderr -o $SCRATCH/stderr -N n${nc}_${fst} ${bin_dir}/mult_refine_n.com $fst $lastn $start $nc 1 0 0 0 1 1`
          endif
        else if (`echo ${cluster_type} | tr '[a-z]' '[A-Z]'` == "LSF") then 
          set pid = cycle_${firstn}_${start}_${nc}
          if ( $sym == "H" || $sym == "HP" ) then
            bsub -J $pid -W 12:00 -q short -R "select[model!=Opteron2216]" -R "rusage[mem=${mem_big}]" -cwd  "$working_directory" -r -e $SCRATCH/stderr -o $SCRATCH/stderr ${bin_dir}/mult_hrefine_n.com $firstn $lastn $start $nc 1 0 0 0 1 1
          else
            bsub -J $pid -W 12:00 -q short -R "select[model!=Opteron2216]" -R "rusage[mem=${mem_big}]" -cwd  "$working_directory" -r -e $SCRATCH/stderr -o $SCRATCH/stderr ${bin_dir}/mult_refine_n.com $firstn $lastn $start $nc 1 0 0 0 1 1
          endif
        else
          if ( $sym == "H" || $sym == "HP" ) then
            ${bin_dir}/mult_hrefine_n.com $fst $lastn $start $nc 1 0 0 0 1 1 &
          else
            ${bin_dir}/mult_refine_n.com $fst $lastn $start $nc 1 0 0 0 1 1 &
          endif
          set pid = $!
        endif
      endif
    else if (${shiftiter} != 0) then
      if (${angleiter} == 0) then
        if (`echo ${cluster_type} | tr '[a-z]' '[A-Z]'` == "SGE") then
          if ( $sym == "H" || $sym == "HP" ) then
            set pid = `qsub $stn -l mem_free=${mem_big}G $nq -cwd -r y -V -terse -e $SCRATCH/stderr -o $SCRATCH/stderr -N n${nc}_${fst} ${bin_dir}/mult_hrefine_n.com $fst $lastn $start $nc 1 1 1 1 0 0`
          else
            set pid = `qsub $stn -l mem_free=${mem_big}G $nq -cwd -r y -V -terse -e $SCRATCH/stderr -o $SCRATCH/stderr -N n${nc}_${fst} ${bin_dir}/mult_refine_n.com $fst $lastn $start $nc 1 1 1 1 0 0`
          endif
        else if (`echo ${cluster_type} | tr '[a-z]' '[A-Z]'` == "LSF") then 
          set pid = cycle_${firstn}_${start}_${nc}
          if ( $sym == "H" || $sym == "HP" ) then
            bsub -J $pid -W 12:00 -q short -R "select[model!=Opteron2216]" -R "rusage[mem=${mem_big}]" -cwd  "$working_directory" -r -e $SCRATCH/stderr -o $SCRATCH/stderr ${bin_dir}/mult_hrefine_n.com $firstn $lastn $start $nc 1 1 1 1 0 0
          else
            bsub -J $pid -W 12:00 -q short -R "select[model!=Opteron2216]" -R "rusage[mem=${mem_big}]" -cwd  "$working_directory" -r -e $SCRATCH/stderr -o $SCRATCH/stderr ${bin_dir}/mult_refine_n.com $firstn $lastn $start $nc 1 1 1 1 0 0
          endif
        else
          if ( $sym == "H" || $sym == "HP" ) then
            ${bin_dir}/mult_hrefine_n.com $fst $lastn $start $nc 1 1 1 1 0 0 &
          else
            ${bin_dir}/mult_refine_n.com $fst $lastn $start $nc 1 1 1 1 0 0 &
          endif
          set pid = $!
        endif
      else if (${angleiter} != 0) then
        if (`echo ${cluster_type} | tr '[a-z]' '[A-Z]'` == "SGE") then
          if ( $sym == "H" || $sym == "HP" ) then
            set pid = `qsub $stn -l mem_free=${mem_big}G $nq -cwd -r y -V -terse -e $SCRATCH/stderr -o $SCRATCH/stderr -N n${nc}_${fst} ${bin_dir}/mult_hrefine_n.com $fst $lastn $start $nc 1 0 0 0 0 0`
          else
            set pid = `qsub $stn -l mem_free=${mem_big}G $nq -cwd -r y -V -terse -e $SCRATCH/stderr -o $SCRATCH/stderr -N n${nc}_${fst} ${bin_dir}/mult_refine_n.com $fst $lastn $start $nc 1 0 0 0 0 0`
          endif
        else if (`echo ${cluster_type} | tr '[a-z]' '[A-Z]'` == "LSF") then 
          set pid = cycle_${firstn}_${start}_${nc}
          if ( $sym == "H" || $sym == "HP" ) then
            bsub -J $pid -W 12:00 -q short -R "select[model!=Opteron2216]" -R "rusage[mem=${mem_big}]" -cwd  "$working_directory" -r -e $SCRATCH/stderr -o $SCRATCH/stderr ${bin_dir}/mult_hrefine_n.com $firstn $lastn $start $nc 1 0 0 0 0 0
          else
            bsub -J $pid -W 12:00 -q short -R "select[model!=Opteron2216]" -R "rusage[mem=${mem_big}]" -cwd  "$working_directory" -r -e $SCRATCH/stderr -o $SCRATCH/stderr ${bin_dir}/mult_refine_n.com $firstn $lastn $start $nc 1 0 0 0 0 0
          endif
        else
          if ( $sym == "H" || $sym == "HP" ) then
            ${bin_dir}/mult_hrefine_n.com $fst $lastn $start $nc 1 0 0 0 0 0 &
          else
            ${bin_dir}/mult_refine_n.com $fst $lastn $start $nc 1 0 0 0 0 0 &
          endif
          set pid = $!
        endif
      endif
    endif
    echo $pid ${data_input}_mult_refine_n_r${nc}.log_${fst}_${lastn} >> $SCRATCH/pid_temp.log

    endif
    @ nc++
  end

#  sleep 2

  if ( $lastn == $last ) then
    goto alignment_done
  endif
  @ firstn = $firstn + $incr
  @ lastn = $lastn + $incr
  if ( $firstn >= $last ) set firstn = $last
  if ( $lastn >= $last ) set lastn = $last
end

alignment_done:

sleep 10

if ( $reschedule != "F" ) then
  if (`echo ${cluster_type} | tr '[a-z]' '[A-Z]'` == "SGE") then
    set qw = `qstat | grep " qw   " | wc -l`
    if ( $qw != 0 ) then
      if ($nq != "" ) then
        qdel `grep ref $SCRATCH/pid_temp.log | awk '{print $1}'`
        \rm $SCRATCH/pid_temp.log
        set nq = ""
        goto subloop
      endif
    endif
  endif
endif

cat $SCRATCH/pid_temp.log >> $SCRATCH/pid.log

set nc = 1
while ( $nc <= $nclass )
  \rm ${data_input}_${start}_r${nc}.par >& /dev/null
  @ nc++
end

set nc = 1
while ( $nc <= $nclass )

  set firstn = $first
  @ lastn = $first + $incr - 1
  if ( $lastn >= $last ) set lastn = $last

checkdone:

  sleep 5
  while ( $firstn <= $last )
    set fst = `grep -v C $SCRATCH/${data_input}_${start}_r${nc}.par_${firstn}_${lastn} | tail -1 | awk '{print $1}'`
    if ( $lastn > $fst ) then
      @ fst++
      grep --binary-files=text "overall score" $SCRATCH/${data_input}_${start}_r${nc}.par_${fst}_$lastn >& /dev/null
      if ( ! $status) then
        grep -v C $SCRATCH/${data_input}_${start}_r${nc}.par_${fst}_$lastn >> $SCRATCH/${data_input}_${start}_r${nc}.par_${firstn}_$lastn
        grep --binary-files=text "overall score" $SCRATCH/${data_input}_${start}_r${nc}.par_${fst}_$lastn >> $SCRATCH/${data_input}_${start}_r${nc}.par_${firstn}_$lastn
      endif
    else
      set fst = $firstn
    endif

    grep --binary-files=text "overall score" $SCRATCH/${data_input}_${start}_r${nc}.par_${firstn}_$lastn >& /dev/null

    if ($status) goto checkdone

    echo "Cycle "$start": particles $firstn to $lastn, ref $nc, finished "`date` >> frealign.log
    set pid = `grep "${data_input}_mult_refine_n_r${nc}.log_${fst}_${lastn}" $SCRATCH/pid.log | awk '{print $1}'`
    sed -i "/$pid/d" $SCRATCH/pid.log
    if ($firstn == $first ) head -65 $SCRATCH/${data_input}_${start}_r${nc}.par_${firstn}_${lastn} | grep --binary-files=text C >> ${data_input}_${start}_r${nc}.par

    grep -v C --binary-files=text $SCRATCH/${data_input}_${start}_r${nc}.par_${firstn}_${lastn} >> ${data_input}_${start}_r${nc}.par
    if ( ! $no_delete ) \rm $SCRATCH/${data_input}_${start}_r${nc}.par_${firstn}_${lastn} >& /dev/null

    @ firstn = $firstn + $incr
    @ lastn = $lastn + $incr
    if ( $lastn >= $last ) set lastn = $last
  end

collect_done:

  @ nc++

end

if ($nclass <= 1) goto skip_occ

echo $nclass > $SCRATCH/calc_occ.in
echo 1.0 >> $SCRATCH/calc_occ.in
set nc = 1
while ($nc <= $nclass)
  echo ${data_input}_${start}_r${nc}.par >> $SCRATCH/calc_occ.in
  @ nc++
end
set nc = 1
while ($nc <= $nclass)
  echo ${data_input}_${start}_r${nc}.par >> $SCRATCH/calc_occ.in
  @ nc++
end
  
time ${bin_dir}/calc_occ.exe < $SCRATCH/calc_occ.in >& $SCRATCH/calc_occ.log

skip_occ:

reconstruct:
echo "Cycle "$start": calculating 3D structure...." >> frealign.log

${bin_dir}/mult_reconstruct.com $first $last $start

# sleep 20

if ($nclass <= 1) goto skip_occ2

echo $nclass > $SCRATCH/calc_occ.in
echo 1.0 >> $SCRATCH/calc_occ.in
set nc = 1
while ($nc <= $nclass)
  echo ${data_input}_${start}_r${nc}.par >> $SCRATCH/calc_occ.in
  @ nc++
end
set nc = 1
while ($nc <= $nclass)
  echo ${data_input}_${start}_r${nc}.par >> $SCRATCH/calc_occ.in
  @ nc++
end
  
time ${bin_dir}/calc_occ.exe < $SCRATCH/calc_occ.in >& $SCRATCH/calc_occ.log

skip_occ2:

cp mparameters $SCRATCH/mparameters_run
set end		= `grep end_process $SCRATCH/mparameters_run | awk '{print $2}'`

if ($start < $end ) then
  @ start++
  goto mainloop
endif

echo "Normal termination of refinement "`date` >> frealign.log
echo "monitor term" >> $SCRATCH/pid.log

date
