C*
C* Copyright 2013 Howard Hughes Medical Institute.
C* All rights reserved.
C* Use is subject to Janelia Farm Research Campus Software Copyright 1.1
C* license terms ( http://license.janelia.org/license/jfrc_copyright_1_1.html )
C*
C**************************************************************************
      REAL FUNCTION CC3(NSAM,IRAD,AMAG,SPEC,SPEQ,A3DF,A3DS,
     +			 MAXR1,MAXR2,PHI,THETA,PSI,SHX,SHY,IBUF,
     +			 PBUC,PBUQ,RBFACT,SINCLUT,IPAD,RBUF,FSCT,
     +                   IEWALD,THETATR,CTFF,CTFS,RI2,RI3,RIH,
     +                   HALFW,PWEIGHTS)
C**************************************************************************
C  Calculates overall score, used to search and refine all parameters
C  Score is SNR weighted so that data near CTF zero does not 
C  affect orientation parameters - similar function is available unweighted.
C  Uses functions PDIFF and AINTERPO3DS or AINTERPO3DBIG.
C  Used in CALCFX, PREFINE and PSEARCH
C**************************************************************************
      IMPLICIT NONE

      INTEGER NSAM,JC,I,J,II,JJ,MAXR1,MAXR2,IRAD,NSAMH,ID,MAXR12
      INTEGER MAXR22,ITEMP,IBUF,IPAD,IBIN,IB1,IB2,IB,IC
      REAL DM(9),PHI,THETA,PSI,CPHI,SPHI,CTHE,STHE,CPSI,SPSI
      REAL PHASE,X3,Y3,Z3,DPHA,SHX,SHY,PDIFF,FSCT(*),W,SNRP
      REAL WGT,AMAG,PI2,RBFACT,SINCLUT(*),RBUF(*),SNRV,R,SCAL
      REAL RI2,RI3,RIH,HALFW,PWEIGHTS(*)
      DOUBLE PRECISION SUM1,SUM2,SUM3
      PARAMETER  (PI2=3.1415926535897/2.0)
      COMPLEX SAMP,SPEC(*),SPEQ(*),PSHFT,AINPO,AINTERPO3DS
      COMPLEX A3DF(*),A3DS(*),AINTERPO3DBIG,PBUC(*),PBUQ(*)
C MW for Ewald sphere corrected extraction
      COMPLEX EWALDEX,CTFF(*),CTFS(*),CTFR,CTFL
      INTEGER IEWALD
      REAL THETATR         ! needed by function EWALDEX
C**************************************************************************
      JC=NSAM/2+1
      IC=NSAM*NSAM/2
      NSAMH=NSAM/2
      W=1.0/REAL(NSAM)
      MAXR12=MAXR1**2
      MAXR22=MAXR2**2
      CPHI=COS(PHI)
      SPHI=SIN(PHI)
      CTHE=COS(THETA)
      STHE=SIN(THETA)
      CPSI=COS(PSI)
      SPSI=SIN(PSI)

      DM(1)=(CPHI*CTHE*CPSI-SPHI*SPSI)/ABS(AMAG)
      DM(2)=(SPHI*CTHE*CPSI+CPHI*SPSI)/ABS(AMAG)
      DM(3)=-STHE*CPSI/ABS(AMAG)
      DM(4)=(-CPHI*CTHE*SPSI-SPHI*CPSI)/ABS(AMAG)
      DM(5)=(-SPHI*CTHE*SPSI+CPHI*CPSI)/ABS(AMAG)
      DM(6)=STHE*SPSI/ABS(AMAG)
C MW next lines uncommented for Ewald sphere correction
      DM(7)=STHE*CPHI/ABS(AMAG)
      DM(8)=STHE*SPHI/ABS(AMAG)
      DM(9)=CTHE/ABS(AMAG)
C
      DO 20 I=1,4*NSAM/2+NSAM*NSAM+2*NSAM
        RBUF(I)=0.0
20    CONTINUE
      IB1=4*NSAM/2
      IB2=IB1+NSAM*NSAM
      SCAL=NSAM*NSAM/2
C
      DO 30 I=0,JC-1
      	II=I+1
      	DO 30 J=-JC+1,JC-1
      	  ITEMP=I**2+J**2
          IF ((ITEMP.GE.MAXR12).AND.(ITEMP.LT.MAXR22)) THEN
      	    JJ=J+1
      	    IF (JJ.LT.1) JJ=JJ+NSAM
      	    IF (II.NE.JC) THEN
      	      ID=II+NSAMH*(JJ-1)
      	      CTFR=CTFF(ID)
      	      CTFL=CTFF(ID+IC)
      	    ELSE
      	      CTFR=CTFS(JJ)
      	      CTFL=CTFS(JJ+NSAM)
      	    ENDIF
C            IF (IEWALD.NE.0) THEN 
C MW call ewald sphere corrected extraction
C              IF (IEWALD.LT.0) THEN
C                CTFR=CONJG(CTFR)
C                CTFL=CONJG(CTFL)
C              ENDIF
C              AINPO=EWALDEX(NSAM,IRAD,A3DF,A3DS,SINCLUT,IPAD,
C     +                         I,J,DM,THETATR,CTFR,CTFL)
C            ELSE 
      	      X3=DM(1)*I+DM(4)*J
      	      Y3=DM(2)*I+DM(5)*J
      	      Z3=DM(3)*I+DM(6)*J
      	      IF (IBUF.GE.0) THEN
                IF (IRAD.EQ.0) THEN
                  AINPO=AINTERPO3DBIG(NSAM,IPAD,A3DF,A3DS,X3,Y3,Z3)
                ELSE
                  AINPO=AINTERPO3DS(IPAD*NSAM,IRAD,A3DF,A3DS,
     +                              X3,Y3,Z3,SINCLUT,IPAD)
                ENDIF
C              ENDIF
              IF (REAL(CTFR+CONJG(CTFL)).LT.0.0)
     +        AINPO=-AINPO
C              AINPO=AINPO*(CTFR+CONJG(CTFL))
            ENDIF
      	    IF (II.NE.JC) THEN
      	      IF (IBUF.GT.0) PBUC(ID)=AINPO
      	      IF (IBUF.LT.0) AINPO=PBUC(ID)
              IB=IB1+2*ID
              RBUF(IB-1)=REAL(AINPO)/SCAL
              RBUF(IB)=AIMAG(AINPO)/SCAL
      	    ELSE
      	      IF (IBUF.GT.0) PBUQ(JJ)=AINPO
      	      IF (IBUF.LT.0) AINPO=PBUQ(JJ)
              IB=IB2+2*JJ
              RBUF(IB-1)=REAL(AINPO)/SCAL
              RBUF(IB)=AIMAG(AINPO)/SCAL
      	    ENDIF
          ENDIF
30    CONTINUE
      CALL RLFT3(RBUF(IB1+1),RBUF(IB2+1),NSAM,NSAM,1,-1)
      CALL MASKCOS2(NSAM,RBUF(IB1+1),RI2,RI3,RIH,HALFW,AMAG)
      CALL RLFT3(RBUF(IB1+1),RBUF(IB2+1),NSAM,NSAM,1,1)
C
      DO 31 I=0,JC-1
      	II=I+1
      	DO 31 J=-JC+1,JC-1
      	  ITEMP=I**2+J**2
          IF ((ITEMP.GE.MAXR12).AND.(ITEMP.LT.MAXR22)) THEN
      	    JJ=J+1
      	    IF (JJ.LT.1) JJ=JJ+NSAM
            PHASE=SHX*I+SHY*J
            PSHFT=CMPLX(COS(PHASE),SIN(PHASE))
      	    IF (II.NE.JC) THEN
      	      ID=II+NSAMH*(JJ-1)
      	      SAMP=SPEC(ID)*PSHFT
              IB=IB1+2*ID
              AINPO=CMPLX(RBUF(IB-1),RBUF(IB))
      	    ELSE
      	      SAMP=SPEQ(JJ)*PSHFT
              IB=IB2+2*JJ
              AINPO=CMPLX(RBUF(IB-1),RBUF(IB))
      	    ENDIF
C
            IBIN=4*(INT(SQRT(FLOAT(ITEMP))+0.5)+1)
            RBUF(IBIN-3)=RBUF(IBIN-3)+REAL(SAMP*CONJG(AINPO))
            RBUF(IBIN-2)=RBUF(IBIN-2)+CABS(SAMP)**2
            RBUF(IBIN-1)=RBUF(IBIN-1)+CABS(AINPO)**2
            RBUF(IBIN)=RBUF(IBIN)+1.0
C
          ENDIF
31    CONTINUE
C
      SUM1=0.0
      SUM2=0.0
      SUM3=0.0
      DO 10 I=2,NSAM/2
        IBIN=4*I
        IF(RBUF(IBIN).NE.0.0) THEN
          R=RBUF(IBIN-3)/SQRT(RBUF(IBIN-1))
          WGT=EXP(-RBFACT*(I-1)**2)
          SUM3=SUM3+R*WGT
          SUM2=SUM2+WGT**2*RBUF(IBIN-2)
          SUM1=SUM1+1.0
        ENDIF
10    CONTINUE
      IF (SUM1*SUM2.NE.0.0) SUM3=SUM3/SQRT(SUM1*SUM2)
      CC3=SUM3
C
      RETURN
      END
